
// Receiving VSI2000 data stream(TCP) and decompose to channel in memory
// Expected data is
// 31_____________________________0
// 11111111111111998877665544332211
// 66554433221100  
// lhlhlhlhlhlhlhlhlhlhlhlhlhlhlhlh
// Note that High/Low bit is opposite order.
// This stream will be converted to 
// t   16151413121110090807060504030201
// ch1 lhlhlhlhlhlhlhlhlhlhlhlhlhlhlhlh
// ch2 lhlhlhlhlhlhlhlhlhlhlhlhlhlhlhlh
// accumurating for 1 second

#ifndef _THREAD_CH_DCON_
#define _THREAD_CH_DCON_

#include <vector>
using namespace std;

extern "C" {
#include <signal.h>
#include <wchar.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <math.h>
}

#include "Thread.h"
#include "Mutex.h"
#include "FileSocket.h"
#include "TCPSocket.h"
#include "payload.h"
#include "netutl.h"

extern char* me;
extern int debug_level;
enum State {NORMAL_STATE,WARN_STATE,STOP_STATE,ACTIVE_STATE};
const int PAGE_READY_WAIT=10;
const int SOCK_READY_WAIT=10;

// #define SHM_DATA ftok("/tmp",1)
#define SHM_DATA 100
#define SHM_PNTR ftok("/tmp",2)
const long nano_wait= 5;

//inline int delete_shm(const int shm_id);

class Recv_mem: public Thread
{
  Payload payld;
  Payload payld_srv;  // Playload for 16 channel data
  int ipage; 
  u_int8_t* page[2]; 
  u_int8_t* w_point; // 
  u_int8_t* r_point; // 
  //  int writing_page;
  //  int reading_page;
  char  page_state[2]; // 0:write OK, 1: read OK 
  Mutex  page_mutex[2];
  int mtu;
  Socket *_in_sock;
  int appl_status; // communicator with application
 protected:
  string srv_ip;
  //  unsigned long duration; // sec
  int srv_port;
  int shift_minus; // bit shit offset
  unsigned long recv_nword; // size for of receving data buffer (unit in 32 bit = 4byte)
  //  int mem_data_id;
  unsigned long mem_data_size; // memory size 
  static int nClientID;
  int so_RcvBufSize;
  string outfile;
public:
  // Recv_mem( Socket& out_sock);
  Recv_mem(const int m );
  ~Recv_mem(){  //Delete_shm();
    Release_data_mem();
  }
  const int Get_ipage()const {return ipage;}
  u_int8_t* Get_Rpoint() const {return r_point;}
  const u_int8_t* Get_Wpoint() const {return w_point;}
  const u_int32_t Get_Pagesize() const {return mem_data_size/2;}
  u_int8_t* Get_page_addr(const int i)const{return page[i];}
  const int Check_forRead(const int i, Socket* s=0);  
  const int Check_forWrite(const int i);  
  const int Read_finsh(const int i);
  const int Write_finsh(const int i);
  void Inc_Rpoint(const u_int32_t d);
  //  void Set_insock(Socket* s){_in_sock=s;}
  void Set_payload(const struct Payload& pld);
  void Set_sample_dur(const u_int32_t sr, unsigned long d){
    payld.clock = sr;payld.length =(unsigned long long)sr*d;
    payld_srv.clock = sr*16;payld_srv.length =(unsigned long long)sr*d*16;
  } // sps
  void Set_bitmask(const u_int32_t bm){
    payld.bit_mask = bm; shift_minus =(int)log2(bm/3);} // sps
  void Set_RecvBufSize(const u_int32_t bs){recv_nword = bs/4;} // sps
  void Set_server(const string ip){srv_ip = ip;}
  //void Set_Duration(const unsigned long long d){payld.length =payld.clock*d;}
  void Set_Srv_port(const int p){srv_port=p;}
  void Print(FILE* fp=stderr);
  int  Setup_data_mem();
  int Delete_shm();
  //  int  Release_data_mem(){if( page[0] !=0){ delete [] page[0]; page[0]=0;}}
  const int  Release_data_mem();
  const int CommunicateWithServer(unsigned long sec,  
				  const string& srv_ip, const int port);
  void* thread_func(void);
  const bool Check_ApplState(const State s){return (appl_status == s);}
  void Set_ApplState(const State s){appl_status = s;}
  void proc();
  void Set_outfile(const string s){outfile=s;}
  void Set_so_rBufSize(const int sb){ so_RcvBufSize = sb;}
  //  void Check_memsize();
//  const int Get_shmid()const {return mem_data_id;}
  //  const int Send_payload();
  const int Wait(const int dur);
private:
  int Ch_decon(u_int32_t* dat,  unsigned long current_byte, unsigned long nbyte);
  long  Ch_decon2(u_int32_t* dat,  long long, unsigned long nbyte);
  void Byte_decon(u_int32_t* dat);
  //  void Byte_decon2(u_int32_t* dat);
};
int Recv_mem::nClientID = 0;

// // Constructor 
// inline Recv_mem::Recv_mem( Socket &out_sock):
//   _in_sock(out_sock),  mem_data_size(0),
//   mem_data_id(0),
//   recv_nword(256), w_point(0),r_point(0),ipage(0)
// {
//   page[0] = page[1] =0;
// }


//Constructor 
inline Recv_mem::Recv_mem(const int m):
  mem_data_size(0), mtu(m),_in_sock(0),
     recv_nword(256), w_point(0),r_point(0),ipage(0),
     srv_port(7000),srv_ip("127.0.0.1"),
  shift_minus(0),so_RcvBufSize(1000000),
  appl_status(NORMAL_STATE)
{
  page[0] = page[1] =0;
  page_state[0] = page_state[1] =0; // write OK;
}

// 
inline void Recv_mem::Set_payload(const struct Payload& pld){
  payld = pld; 
  payld_srv= pld;
  shift_minus=(int)log2(pld.bit_mask/3);
  payld_srv.length=payld.length*16;
  payld_srv.clock=payld.clock*16;
}


inline  void Recv_mem::proc(){
  for(int i=0; i<2; i++){
    page_mutex[i].lock(); // wait until data is ready
    page_state[i] =0;
    page_mutex[i].unlock();
  }
  if(nClientID ==0) start();
  else fprintf(stderr,"Error: Receiver is already running\n");
}


inline const int Recv_mem::Check_forRead(const int i, Socket* out_sock){
  struct timespec req={0,nano_wait}; // 5 nano sec
  
  if( i != 0 && i !=1){
    fprintf(stderr,"Check_forRead(): Error %d not 0 or 1\n", i);
    return -1;
  }
  int scnt=0, pcnt=0;
  while(scnt < SOCK_READY_WAIT && 
	pcnt < PAGE_READY_WAIT ){
//     if( _inexc_sock==0){
// 	usleep(1);
// 	scnt++;
//  	continue;
//       }
      if( page[0]==0){
	usleep(1);
	pcnt++;
 	continue;
      }
//       if(_in_sock->select(Socket::SEL_EXCEPT,1.e-6)){
// 	fprintf(stderr,"Exception _in_sock, probably closed.\n");
// 	return -2; // output last once and finish
//       }
      if(out_sock != 0 && 
	 //	 out_sock->select(Socket::SEL_EXCEPT | Socket::SEL_WRITE,1.e-6)){
	 out_sock->select(Socket::SEL_EXCEPT ,1.e-6)){
	fprintf(stderr,"Exception out_sock, probably closed.\n");
	Set_ApplState(STOP_STATE);
	return -3;  // Finish immediately 
      }
      if( page_state[i]==1) 	break; // read OK
      nanosleep(&req,0);
      if(debug_level > 1)
	fprintf(stderr,"\rNot ready for read(%d) %d %d", 
		i, page_state[0], page_state[1]);
    }
  if(scnt >= SOCK_READY_WAIT){
    fprintf(stderr,
	    "Check_forRead:Too much waiting(%d us) for Socket for data source\n", 
	    SOCK_READY_WAIT);
    return -1;
  }
  if(pcnt >= PAGE_READY_WAIT){
    fprintf(stderr,"Check_forRead:Too much waiting (%d us) for Memory for data source\n",
	    PAGE_READY_WAIT);
    return -1;
  }

  if( debug_level > 0)
    fprintf(stderr,"Page(%d) Ready for Read...",i);
  return 0;


}

inline const int Recv_mem::Check_forWrite(const int i){
  struct timespec req={0,nano_wait}; // 5 nano sec
 
  if( i != 0 && i !=1){
    fprintf(stderr,"Check_forWrite(): Error %d not 0 or 1\n", i);
    return -1;
  }

  int scnt=0, pcnt=0;
  while(scnt < SOCK_READY_WAIT && 
	pcnt < PAGE_READY_WAIT ){
    if( _in_sock==0){
      usleep(1);
      scnt++;
      continue;
    }
    if( page[0]==0){
      usleep(1);
      pcnt++;
      continue;
    }
    if(_in_sock->select(Socket::SEL_EXCEPT,1.e-6)){
      fprintf(stderr,"Exception _in_sock, probably closed.\n");
      return -2;
    }
    if( page_state[i]==0 || Check_ApplState(STOP_STATE)) break; // if writable or stop
    nanosleep(&req,0);
    if(debug_level > 1)
      fprintf(stderr,"\rNot ready for write(%d) %d %d", 
	      i, page_state[0], page_state[1]);
    //      else fprintf(stderr,"page(%d) is still locked\n",i);
  }
  
  if(scnt >= SOCK_READY_WAIT){
    fprintf(stderr,"Check_forWrite:Too much waiting(%d us) for Socket for data source\n", 
	    SOCK_READY_WAIT);
    return -1;
  }
  if(pcnt >= PAGE_READY_WAIT){
    fprintf(stderr,"Check_forWrite:Too much waiting (%d us) for Memory for data source\n",
	    PAGE_READY_WAIT);
    return -1;
  }

  if(Check_ApplState(STOP_STATE)){
    fprintf(stderr,"Check_forWrite:Flag 'Data Reading Application stopped' sensed,state=%d\n"
	    ,appl_status);
    fprintf(stderr,"quit to avoid dead lock\n");
    return -1;
  }
  
  if( debug_level > 0)
    fprintf(stderr,"Page(%d) Ready for write...",i);
  return 0;
}


inline const int Recv_mem::Read_finsh(const int i){
  if( i == 0 || i ==1){
    page_mutex[i].lock();
    page_state[i]=0; //writable
    page_mutex[i].unlock();
    if( debug_level > 0)  fprintf(stderr,"read Finish\n");
    return 0;
  }else{
    fprintf(stderr,"Read_finish(): Error %d not 0 or 1\n", i);
    return -1;
  }
}

inline const int Recv_mem::Write_finsh(const int i){
  if( i == 0 || i ==1){
    page_mutex[i].lock();
    page_state[i]=1; //readable
    page_mutex[i].unlock();
    if( debug_level > 0)  fprintf(stderr,"write Finish\n");
    return 0;
  }else{
    fprintf(stderr,"Write_finish(): Error %d not 0 or 1\n", i);
    return -1;
  }
}

inline void Recv_mem::Print(FILE *fp){
  if(_in_sock !=0) _in_sock->Print(fp);
  fprintf(fp," Config of Recv_mem data------------\n");
  fprintf(fp, "Memory size=%.3f MB\n", (float)mem_data_size*1e-6);
  fprintf(fp, "Srv_ip=%s:%d \n", srv_ip.c_str(), srv_port);
  fprintf(fp, "shift minus=%d\n", shift_minus);
  payld.Print(fp);
  fprintf(fp," -------------------------------------\n");
}

// Get Shared memory for data
inline int Recv_mem::Setup_data_mem(){
  // for 2 pages for in and out
  mem_data_size = payld.clock*payld.Nbits()/8*2; // in Byte for 2 bit, 2pages 
  if(page[0] !=0) delete page[0];
  page[0] = new u_int8_t[mem_data_size];
  fprintf(stderr,"Memory data size=%ld allocated\n",mem_data_size);
  

//   if((mem_data_id=shmget(SHM_DATA,mem_data_size,IPC_CREAT|0666))==-1){
//     if(errno == EEXIST){
//       fprintf(stderr,"ShemID_data=%d already exist\n", mem_data_id);
//       perror("shmget for data");
//     }else{
//       perror("shmget for data");
//       return(-1);
//     }
//   } 
//   if((page[0] = (u_int8_t*)shmat(mem_data_id,0,0)) == (u_int8_t*)-1){
//     fprintf(stderr,"    : Shard Memory Attach Error ");
//     perror("shmat for data");
//     return -1;
//   }
  // Pointer initialize 
  page[1] = page[0] + mem_data_size/2;
  r_point = w_point = page[0];
  //  fprintf(stderr,"Memory(%.3f MB 2 pages) successed\n",   (float)mem_data_size/2000000.);
  //  fprintf(stderr,"Shared memID(%ld MB 2 pages) = %d successed\n",  mem_data_size/2000000, mem_data_id);
  return 0;
}

const int Recv_mem::Release_data_mem(){

  //   int ret = shmdt(page[0]);
  //   if(ret==0) fprintf(stderr,"Shared Memory (id=%d, size=%.3f MB) detached\n",
  // 		     mem_data_id, (float)mem_data_size*1.e-6);
  if(page[0] !=0) delete [] page[0];
  mem_data_size=0;
  fprintf(stderr,"Memory (size=%.3f MB) released\n", 
	  (float)mem_data_size*1.e-6);
  page[0] = page[1] = 0;
  return 0;
}


// Communications with Client 
inline const int Recv_mem::CommunicateWithServer(unsigned long sec, 
					     const string& srv_ip,
					     const int port){
  
  long long byte_sec = (long long)mem_data_size/2; 
  //  const bsize = recv_nword * 4;
  //  u_int8_t sRecvBuf[mtu1];  // Хåե
  //  u_int8_t sSendBuf[mtu2];  // Хåե
  //  u_int8_t sSendBuf[256];  // Хåե
  u_int8_t sRecvBuf[recv_nword*4];  // Хåե

  fprintf(stderr,"DEBUG: CommunicateWithServer Started\n");   
  signal(SIGTSTP, SIG_IGN);
  Print();

  _in_sock = new TCPSocket(mtu);
  // Bind
/*   char name[80]; */
/*   gethostname(name,80); */
/*   if(_in_sock->bind(name,0) == -1){ */
/*     fprintf(stderr,"%s _in_sock->bind(): Error\n",me); */
/*     perror("sock->bind"); */
/*     if(_in_sock->get_sockd() != -1) _in_sock->close(); */
/*     Set_ApplState(STOP_STATE); */
/*     delete _in_sock; _in_sock=0; */
/*     return -1; */
/*   } */

  _in_sock->set_so_rcvbuf(so_RcvBufSize);

//   if(_in_sock->get_sockd() != -1){
//      _in_sock->close();
//    }
//   if(_in_sock->bind("0.0.0.0",0) == -1){
//     fprintf(stderr,"%s in_sock->bind(): Error\n",me);
//     _in_sock->close();
//     exit(1);
//   }
  fprintf(stderr, "Connection to server ...");
  if( _in_sock->connect(srv_ip, port) == -1){
    fprintf(stderr,"%s sock->connection(): Error\n",me);
    perror("sock->connect:");
    fprintf(stderr,"server = %s port=%d\n",srv_ip.c_str(), port);
    //    _in_sock->Print();
    if(_in_sock->get_sockd() != -1) _in_sock->close();
    exit(1);
    //    return -1;
  }else{
    fprintf(stderr, "Connected\n");
  }

  // Print Current contents of the data 
  //  fprintf(stderr,"---Configuraion is -------------\n");
  Print();
  //  fprintf(stderr,"--------------------------------\n");
  fprintf(stderr,"---Sendig Payload  is-----------\n");
  payld.Print();
  fprintf(stderr,"--------------------------------\n");
  // Sending Payload 
  int ret;
  if(ret = _in_sock->send((u_int8_t *)&payld, sizeof(Payload)) <= 0){
    if( ret < 0){
      fprintf(stderr, "Send_payload Error: send failed \n");
    }else if(ret ==0){
      fprintf(stderr, "Send_payload Error: Socket already closed\n");
    }
    perror("Send_payload Error");
    Set_ApplState(STOP_STATE);
    _in_sock->close();
    delete _in_sock; _in_sock=0;
    return -1;
  }
  fprintf(stderr,"Payload(%d bytes) succesfully sent to server\n", 
	  sizeof(Payload));

  fprintf(stderr,"--------------------------------\n");

  time_t dur = payld.length/payld.clock; //sec
  if(Wait(dur) < 0){
    _in_sock->close();
    fprintf(stderr," Connection Closed\n");
    Set_ApplState(STOP_STATE);
    delete _in_sock; _in_sock=0;
    return -1;
  }
  
  int nsize=0;
  long long byte_cnt=0;
  long long carry_byte=0;
  unsigned long long total_byte=0;
  double factor = payld.Factor();
  unsigned long rcv_byte_sec= (unsigned long)(byte_sec*factor);
  long long status;
  //  int dfd = open("dbg-log",O_WRONLY|O_CREAT);
  for(int isec=0; isec<sec; isec++){
    ipage = isec %2;
    fprintf(stderr, "isec=%d: writing to page (%d)\n", isec, ipage);
    //    fprintf(stderr, "%d", ipage);
    if(Check_forWrite(ipage) < 0) { 
      fprintf(stderr,"Check_forWrite Error\n");
      isec=sec; break;}
    long long remain_byte=byte_sec - carry_byte; // net contents is counted
    //    byte_cnt =carry_byte;
    //    w_point = page[ipage];
    u_int8_t* rec_adr = sRecvBuf+carry_byte;
    //    fprintf(stderr,"reading while bytesec=%Ld,> byte_cnt(carry byte)=%Ld\n",  byte_sec, byte_cnt);
    int err_cnt=0;
    while(byte_cnt < byte_sec){
      status = _in_sock->recv(rec_adr,
			      Min((unsigned long)(remain_byte*factor),
				  (unsigned long)(sizeof(sRecvBuf)-carry_byte)));
      //      fprintf(stderr,"received status= %Ld\n",status);
      if(status ==0){
	if(err_cnt>10){
	  fprintf(stderr,"Connection is closed by Server.\n");
	  isec=sec;	  break;
	}else{
	  err_cnt++; continue;
	}
      }
      if(status < 0){
	if( errno ==0)
	  fprintf(stderr,"Error: Connection was closed by server\n"); 
	  else
	    fprintf(stderr,"Error in recv() (errno=%d):%s \n", 
		    errno, strerror(errno));
	Set_ApplState(STOP_STATE);
	isec=sec;	
	break;
      }
      unsigned long ndata = status + carry_byte;
//       if(ndata % 64 !=0){
// 	fprintf(stderr,"Error: Obtained byte = %d \% 64 = %d. != 0\n",
// 		status, status%4);
// 	isec = sec;	break;
//       }
      err_cnt=0;
      //      carry_byte = Ch_decon((u_int32_t *)sRecvBuf, byte_cnt, ndata);
      carry_byte = Ch_decon2((u_int32_t *)sRecvBuf, byte_cnt, ndata);

      byte_cnt += (unsigned long)(ndata /64)*4;
      remain_byte -= (long long)(ndata/64)*4;
      // fputc('.',stderr);
      //      fprintf(stderr,"Byte_cnt = %ld < %ld\n", byte_cnt, byte_sec);
      //      usleep(500000); // wait .5 sec
      total_byte+=status;
    }// while loop;
    if(debug_level > 2)
      fprintf(stderr,"remain loop finished\n");    

    if(byte_cnt >= byte_sec){
      Write_finsh(ipage);
      byte_cnt -= byte_sec;
    }else {
      fprintf(stderr,"Remaining %Ld Byte, page_status unchanged\n",
	      remain_byte);
    }
    
    //    write(dfd,page[isec%2], byte_sec);
  } // for sec 
  //  close(dfd);
  
  fprintf(stderr,"\nTotal (%.3f MB) Received from Server\n",
	  (float)total_byte*1.e-6);
  //  if(page_state[0] + page_state[1] >0) usleep(10);
  while(!Check_ApplState(STOP_STATE)) usleep(100);
  fprintf(stderr,"Connection for Server Closed\n");
  if(_in_sock->get_sockd() != -1) _in_sock->close();
   delete _in_sock;
   _in_sock=0;
  return 0;
}


inline void Recv_mem::Byte_decon(u_int32_t* dat){
  u_int32_t *buf = (u_int32_t*)w_point;
  //  u_int32_t sum=0;
  //  int ich = shift_minus/2;
  *buf=0;
  for(int shift2=0; shift2<32; shift2 += 2) 
    *buf|=(((dat[shift2/2] >>shift_minus) &0x03) << shift2);
  w_point+=4;
  
}

// inline void Recv_mem::Byte_decon2(u_int32_t* dat){
//   u_int32_t buf;
  
//   int ich = shift_minus/2;
//   buf = (*dat & payld.bit_mask) >> shift_minus;  
//   dat++;
//   for(int i=1; i<ich; i++) {  
//     buf |= ( *dat & payld.bit_mask) >> (shift_minus-2*i); dat++;}
//   for(int i=ich; i<16; i++) {  
//     buf |= ( *dat & payld.bit_mask) << (2*i-shift_minus); dat++;}

//   fprintf(stderr,"%0lX ", buf);
//   //  *w_point=sum;
  
//   //  w_point += 4;  // 4bytes
// }


inline int Recv_mem::Ch_decon(u_int32_t* dat, 
				   unsigned long current_byte, 
				   unsigned long nbyte){
  u_int8_t* page_end = page[ipage]+mem_data_size/2;
  
  unsigned long remain = mem_data_size/2 - current_byte;
  unsigned long carry=0;
  int net_data = nbyte/16;
  if( remain >= net_data){
    for(int iword=0; iword<net_data/64; iword++){ 
      Byte_decon(dat);
//       Byte_decon2(dat);
//       fprintf(stderr,"\n");
//       for(int i=0; i<4; i++){
// 	for(int j=0; j<4; j++)
// 	  fprintf(stderr,"%0lX ", dat[j+i*4]);
// 	fputc('\n',stderr);
//       }
      dat+=16; }
    //    fputc('\n',stderr);
    return 0;
  }else{
    for(int iword=0; iword<remain/64; iword++){   Byte_decon(dat);dat+=16;   }
    Write_finsh(ipage);
    int jpage = (ipage +1)  %2;
    if(Check_forWrite(jpage) < 0){
      fprintf(stderr,"Check_forWrite Error\n");
      return -1; }
    w_point = page[jpage];
    carry  = (net_data - remain);
    int four_word = carry/64;
    for(int iword=0; iword<four_word; iword++){   Byte_decon(dat);dat+=16;   }
    return carry;
  }
}

inline long Recv_mem::Ch_decon2(u_int32_t* dat, long long byte_cnt,
					 unsigned long nbyte){
  u_int32_t* pb0 = dat;
  //  long rspace = mem_data_size/2 - (w_point - page[ipage]);
  long rspace = mem_data_size/2 - byte_cnt;
  int net_data = (nbyte/64)*4;  
  if(rspace %4 !=0){
    fprintf(stderr, 
	    "Chdecon2 Error: Remaining space(%ld) is not multile of 4 bites\n",
	    rspace);
    exit(1);
  }
  
  if( rspace >= net_data){
    while(nbyte >=64){
      Byte_decon(dat);dat +=16;
      nbyte -= 64; rspace -=4;
    }      
    //    fprintf(stderr,"w_pint=%0lX ", *(w_point-4));
    if (rspace ==0){
//      Write_finsh(ipage);
//      ipage= 
      w_point = page[(ipage +1)%2];

    }
    return nbyte;
  }else{
    while(rspace >= 4){
      Byte_decon(dat);dat +=16;
      nbyte -=16;   rspace -=4;
    }      
    fprintf(stderr,"w_pint=%0lX ", *(w_point-4));
    if(rspace < 0){
      fprintf(stderr, 
	      "Chdecon2 Error: Remaining space(%ld) is not 4 X n bites\n",
	      rspace);
      exit(1);
    }
//    Write_finsh(ipage);
    rspace = mem_data_size/2;
    int jpage = (ipage +1)  %2;
    fprintf(stderr,"Page %d -> %d \n", ipage, jpage);
    if(Check_forWrite(jpage) < 0){
      fprintf(stderr,"Check_forWrite Error\n");
      return -1; }
    w_point = page[jpage];
    //    ipage=jpage;
    while(nbyte >=64){
      Byte_decon(dat);dat +=16;
      nbyte -= 64; rspace -=4;
    }      
    fprintf(stderr,"w_pint=%0lX ", *(w_point-4));
    if (rspace ==0) Write_finsh(ipage);
    memcpy(pb0,dat,nbyte);
    return nbyte;
  }
}

// Thread 
inline void* Recv_mem::thread_func(void){
  fprintf(stderr,"--Starting Recv_mem thread\n");
  nClientID++; 
  Set_ApplState(NORMAL_STATE);
  // Shard Memory  Initialize
  if( Setup_data_mem() < 0 ){
    //      ||  recv_mem.Setup_prt_shm() < 0){
    fprintf(stderr,"thread_func teminated due to shmget Error");
    exit(1);
  }else{
    long dur = (payld.length / (unsigned long long)payld.clock);
    CommunicateWithServer(dur, srv_ip, srv_port);
    Release_data_mem();
  }
  
  nClientID--;
    fprintf(stderr,"--Closing Recv_mem thread\n");
}

inline const int Recv_mem::Wait(const int dur){
  
  time_t t_now = time(0), t1=payld.epoch;
  char time_code[80], time_code_st[80];  
  strftime(time_code_st,sizeof(time_code_st),"%Y/%j-%H:%M:%S",gmtime(&t1));
  if( payld.epoch + dur < t_now){ 
    fprintf(stderr,"Error: Requested time is already passed\n");
    strftime(time_code,sizeof(time_code),"%Y/%j-%H:%M:%S",gmtime(&t_now));
    fprintf(stderr," Current time = %s\n",time_code);
    return -1;
  }
  
  if(payld.epoch - 1 >= t_now){
    fprintf(stderr,"Waiting until start time %s\n", time_code_st);
    while( payld.epoch -1 > time(0)){
      t_now = time(0);
      strftime(time_code,sizeof(time_code),"%Y/%j %H:%M:%S",gmtime(&t_now));
      fprintf(stderr,"\r Current time = %s  Target time=%s\r",time_code,time_code_st);
      if(Check_ApplState(STOP_STATE)) return -1;
      usleep(300000);
    }
  }  
  return 0;
}


/* inline void Recv_mem::Check_memsize(){ */
/*   unsigned long new_msize = payld.clock*payld.Nbits()/8*2; // in Byte for 2 bit, 2pages  */
/*   if(new_msize > mem_data_size){ */
/*      Setup_data_mem(); */
/*      if(debug_level >0) fprintf(stderr,"Memory is updated\n"); */
/*   } */
/* } */


// inline const int Recv_mem::Send_payload(){
//   int ret;
  
//   if( (_in_sock->connect(srv_ip, srv_port) ==-1 )){
//     if(errno != EISCONN){
//       perror("Error in Connection to data server");
//       return -1;
//     }else{
//       perror("Send_payload connect");
//     }
//   }
  
//   if(ret = _in_sock->send((u_int8_t *)&payld, sizeof(Payload)) <= 0){
//     if( ret < 0){
//       fprintf(stderr, "Send_payload Error: send failed \n");
//     }else if(ret ==0){
//       fprintf(stderr, "Send_payload Error: Socket already closed\n");
//     }
//     perror("Send_payload Error");
//   }
  
//   fprintf(stderr,"Payload(%d bytes) succesfully sent to server\n", 
// 	  sizeof(Payload));
//   return 0;
// }

inline 
const int Read_out(Recv_mem& recv_clt, int isec){
  //  usleep(50);
  int ret=0;
  for(int cnt=0; cnt<isec; cnt++){
    ret = recv_clt.Check_forRead(cnt % 2);
    switch(ret){
    case -1:
      fprintf(stderr,"Memory page (%d)is not available\n", cnt%2);
      return -1;  break;
    case -2:
      fprintf(stderr,"Send out last once\n");
      cnt=isec; break;
    }
    u_int8_t* rdadr = recv_clt.Get_Rpoint();
    if(rdadr ==0){
      fprintf(stderr,"Read address is not ready\n");
      return -1;
    }
    u_int32_t* buf= (u_int32_t*)rdadr;
    fprintf(stderr, "  Read Out data %2d-------\n", cnt);
    for(int j=0; j<4; j++){
      for(int i=0; i<4;i++,buf++) fprintf(stderr, "%0X ",*buf );
      fputc('\n',stderr);
    }
  }
  return 0;
}

// inline int Recv_mem::Delete_shm(){
//   int ret = 0;
//   if(ret = shmctl(mem_data_id,IPC_RMID,0)==-1 )
//     perror("shmctl:remove shem_ptr_id");
  
//   if(ret==0){
//     fprintf(stderr,"Shared memory(id=%d) %ld MB is released\n", 
// 	    mem_data_id, mem_data_size/1000000);
//   }
//   return ret;
// }

inline
void StrUpper(char *str, int n){
  int i;
  for(i=0; i<n; i++) str[i] = toupper((int)str[i]);
}


#endif
