
// Receiving VSI2000 data stream(TCP) and decompose to channel in memory
// Expected data is
// 31_____________________________0
// 11111111111111998877665544332211
// 66554433221100  
// lhlhlhlhlhlhlhlhlhlhlhlhlhlhlhlh
// Note that High/Low bit is opposite order.
// This stream will be converted to 
// t   16151413121110090807060504030201
// ch1 lhlhlhlhlhlhlhlhlhlhlhlhlhlhlhlh
// ch2 lhlhlhlhlhlhlhlhlhlhlhlhlhlhlhlh
// accumurating for 1 second

#ifndef _THREAD_CH_DCON_
#define _THREAD_CH_DCON_

#include <vector>
using namespace std;

extern "C" {
#include <signal.h>
#include <wchar.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <math.h>
}

#include "Thread.h"
#include "Mutex.h"
#include "FileSocket.h"
#include "TCPSocket.h"
#include "payload.h"

extern char* me;
#define SHM_DATA ftok("/tmp",1)
#define SHM_PNTR ftok("/tmp",2)

//*** Note: For using, this object must be declared as static ***//
class Recv_mem: public Thread
{
  Payload payld;
  Payload payld_srv;  // Playload for 16 channel data
  int ipage; 
  u_int8_t* page[2]; 
  u_int8_t* w_point; // 
  u_int8_t* r_point; // 
  //  int writing_page;
  //  int reading_page;
  Mutex  page_mute[2];
protected:
  string srv_ip;
  //  unsigned long duration; // sec
  int srv_port;
  Socket& _in_sock;
  int shift_minus; // bit shit offset
  unsigned recv_nword; // size for of receving data buffer (unit in 32 bit = 4byte)
  int mem_data_id;
  unsigned long mem_data_size; // memory size 
  static int nClientID;
public:
  Recv_mem( Socket& out_sock);
  ~Recv_mem(){ 
    if (page[0] != 0){
      delete [] page[0];
      r_point = w_point = page[1]=0;
      fprintf(stderr,"Data Memory (%ld MB) is released \n",mem_data_size/1000000);
    }
  }
  const int Get_ipage()const {return ipage;}
  u_int8_t* Get_Rpoint() const {return r_point;}
  const u_int8_t* Get_Wpoint() const {return w_point;}
  const u_int32_t Get_Pagesize() const {return mem_data_size/2;}
  const int Lock_forRead(const int i);
  void Inc_Rpoint(const u_int32_t d);
  void Set_insock(Socket& s){_in_sock=s;}
  void Set_payload(const struct Payload& pld);
  void Set_sample_dur(const u_int32_t sr, unsigned long d){
    payld.clock = sr;payld.length =(unsigned long long)sr*d;
    payld_srv.clock = sr*16;payld_srv.length =(unsigned long long)sr*d*16;
  } // sps
  void Set_bitmask(const u_int32_t bm){
    payld.bit_mask = bm; shift_minus =(int)log2(bm/3);} // sps
  void Set_RecvBufSize(const u_int32_t bs){recv_nword = bs/4;} // sps
  void Set_server(const string ip){srv_ip = ip;}
  //void Set_Duration(const unsigned long long d){payld.length =payld.clock*d;}
  void Set_Srv_port(const int p){srv_port=p;}
  void Print(FILE* fp=stderr);
  int  Setup_data_mem();
  int  Release_data_mem(){
    if( page[0] !=0){ delete [] page[0]; page[0]=0;
    fprintf(stderr,"Data Memory (%ld MB) is released \n",mem_data_size/1000000);
    }}
  const int CommunicateWithServer(unsigned long sec,  
				  const string& srv_ip, const int port);
  void* thread_func(void);
  void proc();
  const int Get_shmid()const {return mem_data_id;}
  //  const int Send_payload();
private:
  int Ch_decon(u_int32_t* dat,  unsigned long current_byte, unsigned long nbyte);
  void Byte_decon(u_int32_t* dat);
};
int Recv_mem::nClientID = 0;

// Constructor 
inline Recv_mem::Recv_mem( Socket& out_sock):
  _in_sock(out_sock),  mem_data_size(0),
  mem_data_id(0),
  recv_nword(256), w_point(0),r_point(0),ipage(0)
			 // ptr(0)
			 //     Srate(32000000), // 32Msps
{;
  page[0] = page[1] =0;
}

// 
inline void Recv_mem::Set_payload(const struct Payload& pld){
  payld = pld; 
  shift_minus=(int)log2(pld.bit_mask);
  payld_srv.length=payld.length*16;
  payld_srv.clock=payld.clock*16;
}


inline  void Recv_mem::proc(){
  page_mute[0].lock(); // wait until data is ready
  //  page_mutex.lock(); // wait until data is ready
  //  writing_page=0;
  //  page_mutex.unlock();
  if(nClientID ==0) start();
  else fprintf(stderr,"Error: Receiver is already running\n");
}


inline const int Recv_mem::Lock_forRead(const int i){
  if( i == 0 || i ==1){
    int jpage = (i+1)%2;
    page_mute[jpage].unlock();
    fprintf(stderr,"page(%d) is unlocked\n",jpage);
    int ret=0;
    while(ret==0){
      if(_in_sock.select(Socket::SEL_EXCEPT,1.e-6)){
	fprintf(stderr,"Exception in_sock, probably closed.\n");
	return -2;
      }
      if( page_mute[i].trylock() ==0) break; 
      //      else fprintf(stderr,"page(%d) is still locked\n",i);
    }
    fprintf(stderr,"Page(%d) Ready\n",i);
    return 0;
  }else{
    fprintf(stderr,"Chk_Page_read(): Error %d not 0 or 1\n", i);
    return -1;
  }
}

inline void Recv_mem::Print(FILE *fp){
  _in_sock.Print(fp);
  fprintf(fp, "Sampling rate: %ld Msps 2bit\n", payld.clock/1000000);
  fprintf(fp, "Memory size=%ld\n", mem_data_size);
  fprintf(fp, "Bit_mask=%0x Receiver Bufsize=%d bytes\n", 
	  payld.bit_mask, recv_nword*4);
  fprintf(fp, "Srv_ip=%s:%d \n", srv_ip.c_str(), srv_port);
}

// Get Shared memory for data
inline int Recv_mem::Setup_data_mem(){
  // for 2 pages for in and out
  mem_data_size = payld.clock*2/8*2; // in Byte for 2 bit, 2pages 
  if(page[0] == 0)
    page[0] = new u_int8_t[mem_data_size];
//   if((mem_data_id=shmget(SHM_DATA,mem_data_size,IPC_CREAT|0666))==-1){
//     if(errno == EEXIST){
//       fprintf(stderr,"ShemID_data=%d already exist\n", mem_data_id);
//       perror("shmget for data");
//     }else{
//       perror("shmget for data");
//       return(-1);
//     }
//   } 
//    if((page[0] = (u_int8_t*)shmat(mem_data_id,0,0)) == (u_int8_t*)-1){
//     fprintf(stderr,"    : Shard Memory Attach Error ");
//     perror("shmat for data");
//     return -1;
//   }

  // Pointer initialize 
  page[1] = page[0] + mem_data_size/2;
  r_point = w_point = page[0];
  fprintf(stderr,"Memory(%.3f MB 2 pages) successed\n",  (float)mem_data_size/2000000.);
  //  fprintf(stderr,"Shared memID(%ld MB 2 pages) = %d successed\n",  mem_data_size/2000000, mem_data_id);
  return 0;
}


// Communications with Client 
inline const int Recv_mem::CommunicateWithServer(unsigned long sec, 
					     const string& srv_ip,
					     const int port){
  
  unsigned long byte_sec = mem_data_size/2; // Byte for 2bit 16 channels 1sec. 
  int status;
  int mtu = _in_sock.get_mtu();
  //  const bsize = recv_nword * 4;
  //  u_int8_t sRecvBuf[mtu1];  // Хåե
  //  u_int8_t sSendBuf[mtu2];  // Хåե
  //  u_int8_t sSendBuf[256];  // Хåե
  u_int8_t sRecvBuf[recv_nword*4];  // Хåե

  fprintf(stderr,"DEBUG: CommunicateWithServer Started\n");   
  signal(SIGTSTP, SIG_IGN);

  fprintf(stderr, "Connection to server ...");
  if( _in_sock.connect(srv_ip, port) == -1){
    fprintf(stderr,"%s sock->connection(): Error\n",me);
    _in_sock.Print();
    _in_sock.close();
    return -1;
  }else{
    fprintf(stderr, "Connected\n");
  }

  // Print Current contents of the data 
  fprintf(stderr,"---Configuraion is -------------\n");
  Print();
  fprintf(stderr,"---Sendig Payload  is-----------\n");
  payld_srv.Print();
  fprintf(stderr,"--------------------------------\n");
  // Sending Payload 
  int ret;
  if(ret = _in_sock.send((u_int8_t *)&payld_srv, sizeof(Payload)) <= 0){
    if( ret < 0){
      fprintf(stderr, "Send_payload Error: send failed \n");
    }else if(ret ==0){
      fprintf(stderr, "Send_payload Error: Socket already closed\n");
    }
    perror("Send_payload Error");
    return -1;
  }
  fprintf(stderr,"Payload(%d bytes) succesfully sent to server\n", 
	  sizeof(Payload));
  fprintf(stderr,"--------------------------------\n");
  
  int nsize=0;
  unsigned long byte_cnt=0;
  unsigned long carry_byte=0;
  unsigned long long total_byte=0;
  for(int isec=0; isec<sec; isec++){
    ipage = isec %2;
    fprintf(stderr, "writing to page (%d)\n", ipage);
    //    fprintf(stderr, "%d", ipage);
    page_mute[(ipage+1)%2].unlock();
    if(page_mute[ipage].trylock() !=0){
      fprintf(stderr,"page%d is already locked\n", ipage);}
    //    else      fprintf(stderr,"page%d is locked\n", ipage);
    byte_cnt =carry_byte;
    w_point = page[ipage];
    while(byte_cnt < byte_sec){
      status = _in_sock.recv(sRecvBuf, sizeof(sRecvBuf));
      if(status ==0){
	fprintf(stderr,"Connection is already closed\n");
	isec=sec;	  break;
      }
      if(status < 0){
	if( errno !=0)
	  fprintf(stderr,"Error in send() (errno=%d):%s \n", 
		  errno, strerror(errno));
	isec=sec;	
	break;
      }

      if(status % 4 !=0){
	fprintf(stderr,"Error: Obtained byte = %d \% 4 = %d. != 0\n",
		status, status%4);
	isec = sec;	break;
      }
      carry_byte = Ch_decon((u_int32_t *)sRecvBuf, byte_cnt, status);
      byte_cnt += status/16;
      //      fputc('.',stderr);
      //      fprintf(stderr,"Byte_cnt = %ld < %ld\n", byte_cnt, byte_sec);
      //      usleep(500000); // wait .5 sec
      total_byte+=status;
    }// while loop;
    fprintf(stderr,"byte_cnt=%ld, carry_byte=%ld\n",byte_cnt,carry_byte);
	  
  } // for sec 
  fprintf(stderr,"Total (%.3f MB) Received from Server\n",
	  (float)total_byte*1.e-6);
  fprintf(stderr,"Connection for Server Closed\n");
  _in_sock.close();
  return 0;
}

inline void Recv_mem::Byte_decon(u_int32_t* dat){
  u_int32_t* buf = (u_int32_t*)w_point;
  *buf = (dat[0] & payld.bit_mask) >> shift_minus;
  //  for(int i=1; i<16; i++) {  
  for(int i=1; i<4; i++) {  
    *buf |= ( *dat & payld.bit_mask) << (2*i-shift_minus); dat++;}
  w_point ++;
}

inline int Recv_mem::Ch_decon(u_int32_t* dat, 
				   unsigned long current_byte, 
				   unsigned long nbyte){
  u_int8_t* page_end = page[ipage]+mem_data_size/2;
  
  unsigned long remain = mem_data_size/2 - current_byte;
  unsigned long carry=0;
  int net_data = nbyte/16;
  if( remain >= net_data){
    // for(int iword=0; iword<net_data/64; iword++){ Byte_decon(dat);dat+=16;}
    for(int iword=0; iword<net_data/16; iword++){ Byte_decon(dat);dat+=4; }
    return 0;
  }else{
 // for(int iword=0; iword<remain/64; iword++){   Byte_decon(dat);dat+=16;   }
    for(int iword=0; iword<remain/16; iword++){   Byte_decon(dat);dat+=4;   }
    page_mute[ipage].unlock(); // release for reading
    int jpage = (ipage +1)  %2;
    page_mute[jpage].lock(); 
    w_point = page[jpage];
    carry  = (net_data - remain);
    //    int four_word = carry/64;
    int four_word = carry/16;
    //for(int iword=0; iword<four_word; iword++){   Byte_decon(dat);dat+=16;}
    for(int iword=0; iword<four_word; iword++){   Byte_decon(dat);dat+=4;   }
    return carry;
  }
}

// Thread 
inline void* Recv_mem::thread_func(void){

  nClientID++; 
  // Shard Memory  Initialize
  if( Setup_data_mem() < 0 ){
    //      ||  recv_mem.Setup_prt_shm() < 0){
    fprintf(stderr,"thread_func teminated due to shmget Error");
  }else{
    long dur = (payld.length / (unsigned long long)payld.clock);
    CommunicateWithServer(dur, srv_ip, srv_port);
    //    Release_data_mem();
  }
  
  nClientID--;

}

inline void Recv_mem::Inc_Rpoint(const u_int32_t d){
  u_int8_t* end= page[ipage] + mem_data_size;
  u_int32_t remain = end - r_point;
  
  if( remain < d ){
    ipage = (ipage +1) % 2;
    page_mute[ipage].lock();
    page_mute[ipage].unlock();
    if(ipage ==1) 
      r_point += d;
    else
      r_point = page[0] + d-remain;
  }else
    r_point += d;
  
}

// inline const int Recv_mem::Send_payload(){
//   int ret;
  
//   if( (_in_sock.connect(srv_ip, srv_port) ==-1 )){
//     if(errno != EISCONN){
//       perror("Error in Connection to data server");
//       return -1;
//     }else{
//       perror("Send_payload connect");
//     }
//   }
  
//   if(ret = _in_sock.send((u_int8_t *)&payld, sizeof(Payload)) <= 0){
//     if( ret < 0){
//       fprintf(stderr, "Send_payload Error: send failed \n");
//     }else if(ret ==0){
//       fprintf(stderr, "Send_payload Error: Socket already closed\n");
//     }
//     perror("Send_payload Error");
//   }
  
//   fprintf(stderr,"Payload(%d bytes) succesfully sent to server\n", 
// 	  sizeof(Payload));
//   return 0;
// }

inline 
const int Read_out(Recv_mem& recv_clt, int isec){
  //  usleep(50);
  int ret=0;
  for(int cnt=0; cnt<isec; cnt++){
    ret = recv_clt.Lock_forRead(cnt % 2);
    switch(ret){
    case -1:
      fprintf(stderr,"Memory page (%d)is not available\n", cnt%2);
      return -1;  break;
    case -2:
      fprintf(stderr,"Send out last once\n");
      cnt=isec; break;
    }
    u_int8_t* rdadr = recv_clt.Get_Rpoint();
    if(rdadr ==0){
      fprintf(stderr,"Read address is not ready\n");
      return -1;
    }
    fprintf(stderr, "  Read Out data %2d-------\n", cnt);
    for(int j=0; j<4; j++){
      for(int i=0; i<4;i++,rdadr+=4){
	fprintf(stderr, "%x%x%x%x ",
		*rdadr, *(rdadr+1), *(rdadr+2), *(rdadr+3) );
      }
      fputc('\n',stderr);
    }
    //    recv_clt.Read_end(cnt % 2);
  }
  return 0;
}

inline int delete_shm(const int shm_id){
  int ret = 0;
  if(ret = shmctl(shm_id,IPC_RMID,0)==-1 )
    perror("shmctl:remove shem_ptr_id");
  if(ret==0){
    fprintf(stderr,"Shared memory(id=%d) released\n", shm_id);
  }
  
  return ret;
}

inline
void StrUpper(char *str, int n){
  int i;
  for(i=0; i<n; i++) str[i] = toupper((int)str[i]);
}


#endif
