/* 
 * This software is released under the terms of the MIT License(included below). 
 *  
 * Copyright (c) 2003 MIT Haystack Observatory 
 *  
 * Permission is hereby granted, free of charge, to any person obtaining a   
 * copy of this software and associated documentation files (the "Software"),    
 * to deal in the Software without restriction, including without limitation   
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,   
 * and/or sell copies of the Software, and to permit persons to whom the  
 * Software is furnished to do so, subject to the following conditions: 
 *  
 * The above copyright notice and this permission notice shall be included in    
 * all copies or substantial portions of the Software. 
 *   
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,  
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE  
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER   
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE 
 * SOFTWARE. 
 * 
 */

#include <unistd.h>
#include <RTCPSocket.h>

RTCPSocket::RTCPSocket(Socket& sock,
		       RTPSession& rtp_session,
		       const int& mtu): 
  _sock(sock),	      //  TCPSocket(sock),
  _rtp_session(rtp_session),
  _mtu(mtu),
  _buf(0)
{
  _buf=new u_int8_t[_mtu];
}


RTCPSocket::RTCPSocket(const RTCPSocket& r):
  _sock(r._sock),
  _rtp_session(r._rtp_session),
  _mtu(r._mtu),
  _buf(0)
{
  _buf=new u_int8_t[_mtu];
  if (memcpy(_buf, r._buf, _mtu)!=_buf) {
    syslog(LOG_ERR, "RTCPSocket: memcpy(): %s",
	   strerror(errno));
    exit(1);
  }
}

RTCPSocket::~RTCPSocket()
{
  delete[] _buf;
}

void RTCPSocket::print_sr(uint8_t* sb, FILE* fp)
{
  struct RTCPPacket* h=(RTCPPacket*)&(sb[0]);

  uint8_t pt=h->h._common._pt;
  u_int16_t length=ntohs(h->h._common._length);
  u_int32_t ssrc=ntohl(h->r.sr._ssrc);
  u_int32_t ntp_sec=ntohl(h->r.sr._ntp_sec);
  u_int32_t ntp_frac=ntohl(h->r.sr._ntp_frac);
  u_int32_t rtp_ts=ntohl(h->r.sr._rtp_ts);	
  u_int32_t psent=ntohl(h->r.sr._psent);	
  u_int32_t osent=ntohl(h->r.sr._osent);	
  fprintf(fp,"<sender_report>\n");
  fprintf(fp,"    <pt> %x </pt>\n", pt);
  fprintf(fp,"    <length> %u </length>\n", length);
  fprintf(fp,"    <ssrc> %u </ssrc>\n", ssrc);
  fprintf(fp,"    <ntp_sec> %u </ntp_sec>\n", ntp_sec);
  fprintf(fp,"    <ntp_frac> %u </ntp_frac>\n", ntp_frac);
  fprintf(fp,"    <rtp_ts> %u </rtp_ts>\n", rtp_ts);
  fprintf(fp,"    <psent> %u </psent>\n", psent);
  fprintf(fp,"    <osent> %u </osent>\n", osent);
  fprintf(fp,"</sender_report>\n");
}

void RTCPSocket::print_rr(uint8_t* sb)
{
	 struct RTCPPacket* h=(RTCPPacket*)&(sb[0]);

	 uint8_t pt=h->h._common._pt;
	 u_int16_t length=ntohs(h->h._common._length);
	 u_int32_t ssrc=ntohl(h->r.rr._rr[0]._ssrc);
	 uint8_t fraction=h->r.rr._rr[0]._fraction;
	 u_int32_t lost=ntohl(h->r.rr._rr[0]._lost);
	 u_int32_t last_seq=ntohl(h->r.rr._rr[0]._last_seq);	
	 u_int32_t jitter=ntohl(h->r.rr._rr[0]._jitter);	
	 u_int32_t lsr=ntohl(h->r.rr._rr[0]._lsr);	
	 u_int32_t dlsr=ntohl(h->r.rr._rr[0]._dlsr);	
	 fprintf(fp,"<receiver_report>\n");
	 fprintf(fp,"    <pt> %x </pt>\n", pt);
	 fprintf(fp,"    <length> %u </length>\n", length);
	 fprintf(fp,"    <ssrc> %u </ssrc>\n", ssrc);
	 fprintf(fp,"    <fraction> %u </fraction>\n", fraction);
	 fprintf(fp,"    <lost> %u </lost>\n", lost);
	 fprintf(fp,"    <last_seq> %u <last_seq>\n", last_seq);
	 fprintf(fp,"    <jitter> %u </jitter>\n", jitter);
	 fprintf(fp,"    <lsr> %u </lsr>\n", lsr);
	 fprintf(fp,"    <dlsr> %u </dlsr>\n", dlsr);
	 fprintf(fp,"</receiver_report>\n");
}

void RTCPSocket::print_nack(uint8_t* sb)
{
	 struct RTCPPacket* h=(RTCPPacket*)&(sb[0]);

	 uint8_t pt=h->h._common._pt;
	 uint8_t fmt=h->h._common._count;
	 u_int16_t length=ntohs(h->h._common._length);
	 u_int32_t sender_ssrc=ntohl(h->r._fb._sender_ssrc);
	 u_int32_t src_ssrc=ntohl(h->r._fb._src_ssrc);

	 fprintf(fp,"<nack>\n");
	 fprintf(fp,"    <pt> %x </pt>\n", pt);
	 fprintf(fp,"    <fmt> %x </fmt>\n", fmt);
	 fprintf(fp,"    <length> %u </length>\n", length);
	 fprintf(fp,"    <sender_ssrc> %u </sender_ssrc>\n", sender_ssrc);
	 fprintf(fp,"    <src_ssrc> %u </src_ssrc>\n", src_ssrc);
	 fprintf(fp,"    <feedback>\n");
	 for (int i=0; i<(length-3); ++i) {
		  u_int16_t pid=ntohs(h->r._fb._fci[i]._nack._pid);
		  u_int8_t blp=ntohs(h->r._fb._fci[i]._nack._blp);
		  fprintf(fp,"    <pid> %u </pid>\n", pid);
		  fprintf(fp,"    <blp> %u </blp>\n", blp);
	 }	
	 fprintf(fp,"    </feedback>\n");
	 fprintf(fp,"</nack>\n");
}

void RTCPSocket::print_ack(uint8_t* sb)
{
	 struct RTCPPacket* h=(RTCPPacket*)&(sb[0]);

	 uint8_t pt=h->h._common._pt;
	 uint8_t fmt=h->h._common._count;
	 u_int16_t length=ntohs(h->h._common._length);
	 u_int32_t sender_ssrc=ntohl(h->r._fb._sender_ssrc);
	 u_int32_t src_ssrc=ntohl(h->r._fb._src_ssrc);

	 fprintf(fp,"<ack>\n");
	 fprintf(fp,"    <pt> %x </pt>\n", pt);
	 fprintf(fp,"    <fmt> %x </fmt>\n", fmt);
	 fprintf(fp,"    <length> %u </length>\n", length);
	 fprintf(fp,"    <sender_ssrc> %u </sender_ssrc>\n", sender_ssrc);
	 fprintf(fp,"    <src_ssrc> %u </src_ssrc>\n", src_ssrc);
	 fprintf(fp,"    <feedback>\n");
	 for (int i=0; i<(length-3); ++i) {
		  u_int16_t pid=ntohs(h->r._fb._fci[i]._ack._pid);
		  u_int8_t r=ntohs(h->r._fb._fci[i]._ack._r);
		  u_int8_t blp=ntohs(h->r._fb._fci[i]._ack._blp);
		  fprintf(fp,"        <pid> %u </pid>\n", pid);
		  fprintf(fp,"        <r> %u </r>\n", r);
		  fprintf(fp,"        <blp> %u </blp>\n", blp);
	 }
	 fprintf(fp,"    </feedback>\n");
	 fprintf(fp,"</ack>\n");
}





