/*
 *    VRClientSocket.cpp
 *    vrtp
 *
 *    Created by David Lapsley on Mon Feb 23 2004.
 *    Copyright (c) 2004 __MyCompanyName__. All rights reserved.
 *
 */

#include <VRClientSocket.h>
#include <Utils.h>
#include <unistd.h>
#include <iostream>
#include <errno.h>

void VRClientSocket::rtp_proc() 
{
	 // Runs in same mode as calling context!
	 int rcvd=0, sent=0;
	 while (_active) {
		  rcvd=_in_sock.recv(_buf, _mtu);
		  // Client keeps receiving provided there are no receive errors.
		  // Client relies on RTCP channel for connection termination.
		  if (rcvd==0) {
				syslog(LOG_ERR, "VRClientSocket::rtp_proc(): zero byte read.");
				_active=false;
				continue;
		  }
		  if (rcvd<0) {
				syslog(LOG_ERR, "VRClientSocket::rtp_proc(): read error. %s",
						 strerror(errno));
				_active=false;
				continue;
		  }
		  if (rcvd!=_mtu) {
				syslog(LOG_ERR, "VRClientSocket::rtp_proc(): short read==%d", rcvd);
		  }
		  sent=_out_sock.send(_buf+HDR_SIZE, rcvd-HDR_SIZE);
		  if (sent==0) {
				syslog(LOG_ERR, "VRClientSocket::rtp_proc(): zero byte send.");
				_active=false;
				continue;
		  }
		  if (sent<0) {
				syslog(LOG_ERR, "VRClientSocket::rtp_proc(): send error. %s",
						 strerror(errno));
				_active=false;
				continue;
		  }
		  if (sent!=(rcvd-HDR_SIZE)) {
				syslog(LOG_ERR, "VRClientSocket::rtp_proc(): short send==%d", sent);
				break;
		  }
	 }
	 syslog(LOG_INFO, "VRClientSocket::rtp_proc(): exiting.");
	 _in_sock.shutdown(SHUT_RDWR);
	 _out_sock.shutdown(SHUT_RDWR);
}

void VRClientSocket::rtcp_proc()
{
  // Start asycnrhonous thread (for RTCP processing).
  Thread::start();
}

void* VRClientSocket::thread_func()
{
  // Do RTCP processing.
  while (_active) {
    syslog(LOG_DEBUG, "VRClientSocket::thread_func(): looping");
    _rtcp_sock.recv_proc();
    _rtcp_sock.send_proc();
    usleep((unsigned int)(_rtcp_interval*1e6));
  }
  return(0);
}

VRClientSocket::VRClientSocket(Socket& in_sock,
			       Socket& out_sock,
			       RTCPSocket& rtcp_sock,
			       const double& tvg_duration,
			       const u_int32_t& mtu, 
			       const double& rtcp_interval):
  VRSocket(in_sock, out_sock, rtcp_sock, tvg_duration, mtu, rtcp_interval)
{
}

VRClientSocket::~VRClientSocket()
{
}
