/*
 *    VRServerSocket.cpp
 *    vrtp
 *
 *    Created by David Lapsley on Mon Feb 23 2004.
 *    Copyright (c) 2004 __MyCompanyName__. All rights reserved.
 *
 */

#include <VRServerSocket.h>
#include <Utils.h>
#include <unistd.h>
#include <iostream>
#include <errno.h>

void VRServerSocket::rtp_proc() 
{
	 // Runs in same mode as calling context!
	 int rcvd=0, sent=0;
	 Timer tvg;
	 tvg.start();
	 while (_active) {
		  rcvd=_in_sock.recv(_buf+HDR_SIZE, _mtu-HDR_SIZE);
		  if (rcvd==0) {
				syslog(LOG_ERR, "VRServerSocket::rtp_proc(): zero byte read.");
				_active=false;
				continue;
		  }
		  if (rcvd<0) {
				syslog(LOG_ERR, "VRServerSocket::rtp_proc(): read error. %s",
						 strerror(errno));
				_active=false;
		  }
		  if (rcvd!=(_mtu-HDR_SIZE)) {
				syslog(LOG_ERR, "VRServerSocket::rtp_proc(): short read==%d", rcvd);
		  }
		  sent=_out_sock.send(_buf, rcvd+HDR_SIZE);
		  if (sent==0) {
				syslog(LOG_ERR, "VRServerSocket::rtp_proc: zero byte send.");
				_active=false;
				continue;
		  }
		  if (sent<0) {
				syslog(LOG_ERR, "VRServerSocket::rtp_proc(): send error. %s",
						 strerror(errno));
				_active=false;
		  }
		  if (sent!=rcvd+HDR_SIZE) {
				syslog(LOG_ERR, "VRServerSocket::rtp_proc(): short send==%d", sent);
		  }
		  tvg.stop();
		  if (tvg.elapsed()>_tvg_duration) {
				cerr << "VRServerSocket::rtp_proc(): TVG duration exceeded\n";
				_active=false;
		  }
	 }
	 syslog(LOG_INFO, "VRServerSocket::rtp_proc(): exiting.");
	 _in_sock.shutdown(SHUT_RDWR);
	 _out_sock.shutdown(SHUT_RDWR);
}

void VRServerSocket::rtcp_proc()
{
  // Start asynchronous thread (for RTCP processing).
  Thread::start();
}

void* VRServerSocket::thread_func()
{
  // Do RTCP processing.
  while (_active) {
    syslog(LOG_DEBUG, "VRServerSocket::thread_func(): looping");
    _rtcp_sock.send_proc();
    _rtcp_sock.recv_proc();
    usleep((unsigned long )(_rtcp_interval*1.e6));
  }
  return(0);
}

VRServerSocket::VRServerSocket(Socket& in_sock,
			       Socket& out_sock,
			       RTCPSocket& rtcp_sock,
			       const double& tvg_duration,
			       const u_int32_t& mtu, 
			       const double& rtcp_interval):
  VRSocket(in_sock, out_sock, rtcp_sock, tvg_duration, mtu, rtcp_interval)
{
}

VRServerSocket::~VRServerSocket()
{
}

